#!/usr/bin/perl
use strict;
use vars qw(@ARGV %ENV $VERSION);
$VERSION = 0.90;
use Digest::MD5  qw(md5 md5_hex md5_base64);
use Mail::MboxParser;

use Data::Dump qw(dump);


# CONFIG FILE PROCESSING
my $config;
{
    # loadin config file
    open (CONFIG_FILE, "< $ARGV[0]") or die "Unable to open config file: '$ARGV[0]";
    my @CONFIG_FILE_LINES = <CONFIG_FILE> or die "Unable to read config file: $!";
    close(CONFIG_FILE);

    # try to load data from config file
    eval('$config = { ' . join('',@CONFIG_FILE_LINES) . ' };') or die "Unable to parse config file: '$@'";
}
#warn dump($config);



# RENICE SELF PROCESS
`/usr/bin/renice $config->{RENICE} $$` if defined($config->{RENICE});

# INIT FILTERS
my %FILTERS = ();
{
    # enumerate available filters
    foreach my $filter_name (keys %{$config->{FILTERS}})
    {
#	warn "Found '$filter_name' in config";
#	warn "Attemp to load filter";
	if(eval("require '$config->{FILTERS_PATH}$filter_name.pm'"))
	{
#	    warn "Filter '$filter_name' loaded";
	    my $filter;
            my $filter_config = $config->{FILTERS}->{$filter_name};
	    if (eval('$filter = ' . $filter_name . '->new($filter_config,\%FILTERS);'))
	    {
#		warn "filter '$filter_name' initalizaed" . dump($filter);
		$FILTERS{$filter_name} = $filter;
	    }
	    else
	    {
		warn "unable to init filter '$filter' $@";
	    };
	}
	else
	{
	    warn "Unable to load filter '$filter_name' due to $@";
	};
    };
};                                                                                                                                                            
#warn dump(\%FILTERS);

# STORING INCOMING MESSAGE TO FILE
# generate random id for mail
my $mbox_id = md5_hex(time() . rand(9999));
# compose path for storing attachments
my $attachments_path = $config->{TEMP_PATH} . $mbox_id;
# save mail from STDIN to file
my $mbox_file = $config->{TEMP_PATH} . $mbox_id . ".eml";
open(MAILMESSAGE,"> $mbox_file") or die "Unable to save file $mbox_file due to $!";
while(<STDIN>)
{
    print MAILMESSAGE $_;
};
close(MAILMESSAGE);

my $mbox_locked = 0;

# DECOMPOSE EMAIL
# open mail box file
my $mb = Mail::MboxParser->new(
    $mbox_file,
    decode => 'ALL',
    parseropts => 
    {
        enable_cache    => 1,
        enable_grep     => 1,
        cache_file_name => $mbox_file . '.cache',
    }
    ) or die "Unable to parse email $!";
# message by message
my $msg_number = 0;
for my $msg ($mb->get_messages)
{
    my $SCORE = 0.0;
    my @FILES = ();
    my @FILTERS_LOG = ();

    # DECOMPOSING
    # create directory for attachments
    `/bin/mkdir -p $attachments_path.$msg_number`;
    # storing attachemnts
#    warn "Storing attachments";
    $msg->store_all_attachments(path => $attachments_path . '.' . $msg_number);
#    warn "Attachment stored";
    # storing body
#    warn "Storing body";
    my $body_file_name = "$attachments_path.body.$msg_number";
    open(BODY,"> $body_file_name") or die "Unable to create body file $body_file_name due to $!";
    print BODY $msg->body($msg->find_body);
    close(BODY);
#    warn "Body stored";
    # building files list
    my $attachments_mapping = $msg->get_attachments;
    for my $attach_file_name (keys %$attachments_mapping)
    {
	my %FILE_DESC = 
	(
	    TYPE	=>	"ATTACH",
	    SYS_NAME	=>	$attach_file_name,
	    MAIL_NAME	=>	$attach_file_name,
	);
        $FILE_DESC{SYS_NAME} =~ s#^(\s)#./$1#;
        $FILE_DESC{SYS_NAME} = $attachments_path . '.' . $msg_number . '/' . $FILE_DESC{SYS_NAME};
	push(@FILES,\%FILE_DESC);
    };
    {
	my %FILE_DESC = 
	(
	    TYPE	=>	"BODY",
	    SYS_NAME	=>	$body_file_name,
	    MAIL_NAME	=>	"BODY",
	);
	push(@FILES,\%FILE_DESC);
    };

    # FILTERING
    foreach my $filter_name (keys %FILTERS)
    {
	my $score;
	my $filter = $FILTERS{$filter_name};

	# check headers
	$SCORE += $score if defined($score = $filter->check_header($msg->header,$ARGV[1],\@FILTERS_LOG));
	# check parts
	foreach my $file (@FILES)
	{
	    $SCORE += $score if defined($score = $filter->check_part($file,\@FILTERS_LOG));
	};
    };

    # preparing logs
    my $LOGS = "";
    foreach my $filter_log (@FILTERS_LOG)
    {
        $filter_log->{COMMENT} =~ s/[\t\n\r]//g;
        $LOGS .= sprintf("\t*%6.1f pmf::%s [%s]\n",$filter_log->{SCORE},$filter_log->{FILTER},$filter_log->{COMMENT});
    };

    # printing REWRITED OR NOT
    if(($config->{REWRITE})&&($SCORE > 0.0))
    {
	$mbox_locked++;
	print $msg->{FROM} . "\n";
	print sprintf($config->{REWRITE}->{HEADER},$ARGV[1]);
	print sprintf(
	    $config->{REWRITE}->{BODY},
	    $ARGV[1],
	    "$ARGV[1].$mbox_id",
	    $LOGS,
	    join('',@{$msg->{HEADER}})
	    );
    }
    else
    {
	print join('',@{$msg->{HEADER}});
	my $STATUS = ($SCORE > 0.0)?"Yes":"No";
	print <<EOH
X-pmf-About: pmf-$VERSION, see http://research.m1stereo.tv/verem/pmf
X-pmf-Score: $SCORE
X-pmf-Status: $STATUS
X-pmf-Log:
$LOGS
EOH
;
	print join('',@{$msg->{BODY}});
    };
    

#    print "________________________________________________\n";
    # from
#    print $msg->{FROM};
    # headers
#    print join('',@{$msg->{HEADER}});


#   print	dump($msg);
#	    push(@FILTERS_LOG,sprintf("\t*%6.1f %s [%s]\n",$score,$filter_name,$comment));
#   dump(\@FILTERS_LOG);
    $msg_number++;
};

# try to store email
#warn "mbox_locked:$mbox_locked, savepath:$config->{SAVE_PATH}";
if(($mbox_locked > 0)&&(defined($config->{SAVE_PATH})))
{
    my $store_cmd = "/bin/mv -f $mbox_file $config->{SAVE_PATH}$ARGV[1].$mbox_id";
#    warn $store_cmd;
    `$store_cmd`;
};

# clear temp
`/bin/rm -f -d -r $attachments_path*`;



